<?php
// switchServer
// (c) schmidi 2013
// www.doh-nuts.at


require_once('switchServer/SwitchServer.php');


	
class SwitchServerXASECO extends SwitchServer {
	var $aseco = false;
	
	function onStartup($aseco) {
		$this->aseco = $aseco;
		
		$this->loadSettings('switchServer.xml');
	}
	
	
	function onSync($aseco) {
		$this->aseco->plugin_versions[] = array('plugin' => 'plugin.switchServer.php', 'author' => 'schmidi', 'version' => '0.4.4');

		$this->game_mode = $aseco->server->gameinfo->mode;
		$this->running = ($aseco->currstatus == 4);
		
		$aseco->client->query('GetSystemInfo');
 	  $info = $aseco->client->getResponse();
 	  $this->title_id = $info['TitleId'];
 
		// servers
		if ($this->server_socket) {	
			$this->debug("added SocketDataServer port: {$aseco->server->port}");
			$this->servers[] = new SocketDataServer('127.0.0.1', $aseco->server->port);		
		}
		if ($this->server_public_socket) {
			$opt = &$this->server_public_socket;
			if (intval($opt["port"]) && $opt["password"]) {
				$this->debug("added PublicSocketDataServer ip: {$opt['ip']}, port: {$opt['port']}, password: {$opt['password']}");
				$this->servers[] = new PublicSocketDataServer($opt["ip"], intval($opt["port"]), $opt["password"]);		
			}
			else {
				$this->debug("ERROR PublicSocketDataServer: no port/password spezified");
			}
		}
		if ($this->server_file) {
			$this->debug("added FileDataServer file: {$this->server_file}");
			$this->servers[] = new FileDataServer($this->server_file);		
		}	
		if ($this->server_shared_mem) {
			$this->debug("added SharedMemDataServer login: {$aseco->server->serverlogin}");
			$this->servers[] = new SharedMemDataServer($aseco->server->serverlogin);		
		}	

		$this->init();
		$this->fetchData();
	}


	function onBeginMap($aseco, $map_item) {
		// if gamemode changed, load new layout
		if ($this->game_mode != $aseco->server->gameinfo->mode) {
			$this->game_mode = $aseco->server->gameinfo->mode;
			$this->setLayout();
			$this->generateXML();
		}

		$this->running = true;
		$aseco->addCall('SendDisplayManialinkPageToLogin', array(implode(',', array_keys($this->user_min)), $this->min_XML, 0, false));
		$aseco->addCall('SendDisplayManialinkPageToLogin', array(implode(',', array_keys($this->user_max)), $this->max_XML, 0, false));
	}
	
	
	function onEndMap($aseco, $data) {
		$this->running = false;
 		$aseco->addCall('SendDisplayManialinkPage', array($this->getScoreXML(), 0, false));
	}
	
	
	function onPlayerConnect($aseco, $player) {
		$this->addMaxUser($player->login);
 		$aseco->addCall('SendDisplayManialinkPageToLogin', array($player->login, $this->max_XML, 0, false));
	}
	
	
	function onPlayerDisconnect($aseco, $player) {
		unset($this->user_min[$player->login]);
		unset($this->user_max[$player->login]);
	}


	function onPlayerInfoChanged($aseco, $player) {
		if ($player['SpectatorStatus'] % 10) {
			$this->addMaxUser($player['Login']);
		}
	}
	

// [0]=PlayerUid, [1]=Login, [2]=Time, [3]=Score, [4]=CheckpointIndex
	function onCheckpoint($aseco, $data) {
  	if ($data[4] == 0 && $this->autohide) {
			$this->addMinUser($data[1]);
	 		$aseco->addCall('SendDisplayManialinkPageToLogin', array($data[1], $this->min_XML, 0, false));
 		}
	}  

	
	function onPlayerFinish($aseco, $fin_item) {
  	if ($this->autorestore) {
			$this->addMaxUser($fin_item->player->login);
	 		$aseco->addCall('SendDisplayManialinkPageToLogin', array($fin_item->player->login, $this->max_XML, 0, false));
	 	}
	}  


	// [0]=PlayerUid, [1]=Login, [2]=Answer
	function onManialinkEvent($aseco, $data) {
		if ($data[2] != $this->openActionId && $data[2] != $this->closeActionId)
			return;
			
		$login = &$data[1];	
			
		switch ($data[2]) {
			case $this->openActionId:
				$this->addMaxUser($login);
 				$aseco->addCall('SendDisplayManialinkPageToLogin', array($login, $this->max_XML, 0, false));
 				break;
				
			case $this->closeActionId:
				$this->addMinUser($login);
		 		$aseco->addCall('SendDisplayManialinkPageToLogin', array($login, $this->min_XML, 0, false));
				break;
		}			
	}
	
	
	function onEverySecond($aseco) {
		if (--$this->tick <= 0) {
			$this->tick = 15;
			
			
	  	// count players/specs   
    	$plrs = 0;
    	$specs = 0;
    	$players = &$aseco->server->players;
    	$players->resetPlayers();
	    while ($player = $players->nextPlayer()) {
	      $player->isspectator ? $specs++ : $plrs++;
	    }
       
    	// serverlimits
	    $aseco->client->query('GetLadderServerLimits');
  	  $limits = $aseco->client->getResponse();
 
 			// remove links
 			$name = $this->stripLinks($aseco->server->name);
 			 			
			$srv = new ServerObj($aseco->server->serverlogin, $name, $this->title_id, $plrs, $aseco->server->maxplay, $specs,
													 $aseco->server->maxspec, $limits['LadderServerLimitMin'], $limits['LadderServerLimitMax']);
			
			$this->updateData($srv);
			$this->fetchData();
			
			$this->generateXML();
			
			if ($this->running) {
				$aseco->addCall('SendDisplayManialinkPageToLogin', array(implode(',', array_keys($this->user_max)), $this->max_XML, 0, false));
			}
		}
	}
	
	
	protected function console($txt) {
		$this->aseco->console_text($txt);
	}
	
}	


$SWITCH = new SwitchServerXASECO();
	
Aseco::registerEvent('onStartup', array($SWITCH, 'onStartup'));
Aseco::registerEvent('onSync', array($SWITCH, 'onSync'));
Aseco::registerEvent('onPlayerConnect', array($SWITCH, 'onPlayerConnect'));
Aseco::registerEvent('onPlayerDisconnect', array($SWITCH, 'onPlayerDisconnect'));
Aseco::registerEvent('onBeginMap', array($SWITCH, 'onBeginMap'));
Aseco::registerEvent('onEndMap', array($SWITCH, 'onEndMap'));
Aseco::registerEvent('onPlayerInfoChanged', array($SWITCH, 'onPlayerInfoChanged'));
Aseco::registerEvent('onCheckpoint', array($SWITCH, 'onCheckpoint'));
Aseco::registerEvent('onPlayerFinish', array($SWITCH, 'onPlayerFinish'));
Aseco::registerEvent('onPlayerManialinkPageAnswer', array($SWITCH, 'onManialinkEvent'));
Aseco::registerEvent('onEverySecond', array($SWITCH, 'onEverySecond'));

?>